@@ -320,39 +320,39 @@ oldlogspline <- function(uncensored, right, left, interval, lbound, ubound,
         nsample[6] <- nsample[6]-1
         if(length(table(sample))<3)stop("Not enough unique values")
         z <- .C("logcensor",
-                as.integer(delete),
-                as.integer(iautoknot),
-                as.double(sample),
-                as.integer(nsample),
-                bd = as.double(xbound),
-                SorC = as.integer(SorC),
+                as.integer(c(delete,0,0,0,0)),
+                as.integer(c(iautoknot,0,0,0,0)),
+                as.double(c(sample,0,0,0,0)),
+                as.integer(c(nsample,0,0,0,0)),
+                bd = as.double(c(xbound,0,0,0,0)),
+                SorC = as.integer(c(SorC,0,0,0,0)),
                 nk = as.integer(nknots),
-                kt = as.double(knots),
-                cf = as.double(c(knots, 0, 0)),
-                as.double(penalty),
-                as.double(sample),
-                as.double(sample),
-                logl = as.double(rep(0, n1 + 1)),
+                kt = as.double(c(knots,0,.0,0,0)),
+                cf = as.double(c(knots,0,.0,0,0)),
+                as.double(c(penalty,0,0,0,0)),
+                as.double(c(sample,0,0,0,0)),
+                as.double(c(sample,0,0,0,0)),
+                logl = as.double(rep(0, n1 + 1+10)),
         PACKAGE = "logspline")
         SorC <- z$SorC  # error messages
         if(SorC[1] == -1 && SorC[28] == 0 && nsample[1]!=nsample[2] && nsample[2]>15){
            SorC <- vector(mode = "integer", length = 35)
            SorC[1] <- 1    # the actual function call
            SorC[17] <- 1
-           z <- .C("logcensor",
-                   as.integer(delete),
-                   as.integer(iautoknot),
-                   as.double(sample),
-                   as.integer(nsample),
-                   bd = as.double(xbound),
-                   SorC = as.integer(SorC),
-                   nk = as.integer(nknots),
-                   kt = as.double(knots),
-                   cf = as.double(c(knots, 0, 0)),
-                   as.double(penalty),
-                   as.double(sample),
-                   as.double(sample),
-                   logl = as.double(rep(0, n1 + 1)),
+        z <- .C("logcensor",
+                as.integer(c(delete,0,0,0,0)),
+                as.integer(c(iautoknot,0,0,0,0)),
+                as.double(c(sample,0,0,0,0)),
+                as.integer(c(nsample,0,0,0,0)),
+                bd = as.double(c(xbound,0,0,0,0)),
+                SorC = as.integer(c(SorC,0,0,0,0)),
+                nk = as.integer(nknots),
+                kt = as.double(c(knots,0,.0,0,0)),
+                cf = as.double(c(knots,0,.0,0,0)),
+                as.double(c(penalty,0,0,0,0)),
+                as.double(c(sample,0,0,0,0)),
+                as.double(c(sample,0,0,0,0)),
+                logl = as.double(rep(0, n1 + 1+10)),
            PACKAGE = "logspline")
         }
         bound <- c(z$bd[2], z$bd[3], z$bd[4], z$bd[5])
@@ -444,6 +444,7 @@ logspline <- function(x, lbound, ubound, maxknots=0, knots, nknots=0,
    mm <- range(data)
    if(!missing(lbound)) mm <- range(c(mm, lbound))
    if(!missing(ubound)) mm <- range(c(mm, ubound))
+   if(!missing(knots) && maxknots>0 && length(knots)>maxknots+1) stop("maxknots < length(knots)-1")
 
    # boundaries
    ilow <- (!missing(lbound)) * 1
@@ -601,7 +602,7 @@ rlogspline <- function(n, fit)
     pp <- runif(n)
     qlogspline(pp, fit)
 }
-dlogspline <- function(q, fit)
+dlogspline <- function(q, fit, log = FALSE)
 {
    if(!inherits(fit, "logspline"))
        stop("fit is not a logspline object")
@@ -610,9 +611,10 @@ dlogspline <- function(q, fit)
     y <- fit$coef.pol[1] + x * fit$coef.pol[2]
     for(i in 1:length(fit$knots)) 
        y <- y + fit$coef.kts[i] * ((abs(x - fit$knots[i]) +x- fit$knots[i])/2)^3
-    y <- exp(y)
-    if(fit$bound[1] > 0) y[x < fit$bound[2]] <- 0
-    if(fit$bound[3] > 0) y[x > fit$bound[4]] <- 0
+    if(fit$bound[1] > 0) y[x < fit$bound[2]] <- -Inf
+    if(fit$bound[3] > 0) y[x > fit$bound[4]] <- -Inf
+    if (!log)
+       y <- exp(y)
     y
 }
 plot.logspline <-function(x, n = 100, what = "d", add = FALSE, xlim, xlab = "", ylab = "", type = "l", ...)
